/**
 * From the OpenGL Programming wikibook: http://en.wikibooks.org/wiki/OpenGL_Programming
 * This file is in the public domain.
 * Contributors: Martin Kraus, Sylvain Beucler
 */

in vec4 v_coord;
in vec3 v_normal;
in vec2 texcoord;

out vec4 position;  // position of the vertex (and fragment) in world space
out vec3 varyingNormalDirection;  // surface normal vector in world space
out vec3 LightDirection_cameraspace;
out vec2 f_texcoord;
out vec4 ShadowCoord;
out vec3 normal;
out float z_linear;
out float depth;
out vec3 ViewRay;

uniform mat4 m, v, p,v_depth, p_depth;
uniform mat3 m_3x3_inv_transp;
uniform vec3 lightPos; // light position in world space

void main()
{
  position = m * v_coord;
  varyingNormalDirection = normalize(m_3x3_inv_transp * v_normal); // surface normal vector in world space

  mat4 mvp = p * v * m;
  gl_Position = mvp * v_coord;
  depth = gl_Position.z/gl_Position.w;

  mat4 biasMatrix = mat4(
	0.5, 0.0, 0.0, 0.0,
	0.0, 0.5, 0.0, 0.0,
	0.0, 0.0, 0.5, 0.0,
	0.5, 0.5, 0.5, 1.0
	);
  mat4 depthMVP = p_depth * v_depth * m;
  mat4 depthBiasMVP = biasMatrix * depthMVP;
  ShadowCoord = depthBiasMVP * v_coord;

  // Vector that goes from the vertex to the light, in camera space
  LightDirection_cameraspace = (v * vec4(lightPos,0)).xyz;

  // The unit vector from the eye to the vertex
  vec4 v_eyespace = v * m * v_coord;  
 // ViewRay = vec3(v_eyespace.x/v_eyespace.w, v_eyespace.y/v_eyespace.w, v_eyespace.z/v_eyespace.w);
  ViewRay = vec3(v_eyespace.x, v_eyespace.y, v_eyespace.z);
  ViewRay = normalize(ViewRay);

  z_linear = v_eyespace.z;

  mat4 mv = v * m;
  mat3 mv_3x3_inv_transp = transpose(inverse(mat3(m)));
  normal = mv_3x3_inv_transp * v_normal;

  f_texcoord = texcoord;
}
